/* ***************************************************************** *
 * Copyright 1998 International Business Machines Corporation. All   *
 * Rights Reserved.                                                  *
 *                                                                   *
 * Please read this carefully.  Your use of this reference           *
 * implementation of certain of the IETF public-key infrastructure   *
 * specifications ("Software") indicates your acceptance of the      *
 * following.  If you do not agree to the following, do not install  *
 * or use any of the Software.                                       *
 *                                                                   *
 * Permission to use, reproduce, distribute and create derivative    *
 * works from the Software ("Software Derivative Works"), and to     *
 * distribute such Software Derivative Works is hereby granted to    *
 * you by International Business Machines Corporation ("IBM").  This *
 * permission includes a license under the patents of IBM that are   *
 * necessarily infringed by your use of the Software as provided by  *
 * IBM.                                                              *
 *                                                                   *
 * IBM licenses the Software to you on an "AS IS" basis, without     *
 * warranty of any kind.  IBM HEREBY EXPRESSLY DISCLAIMS ALL         *
 * WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING,   *
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF       *
 * MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A PARTICULAR    *
 * PURPOSE.  You are solely responsible for determining the          *
 * appropriateness of using this Software and assume all risks       *
 * associated with the use of this Software, including but not       *
 * limited to the risks of program errors, damage to or loss of      *
 * data, programs or equipment, and unavailability or interruption   *
 * of operations.                                                    *
 *                                                                   *
 * IBM WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES OR FOR ANY SPECIAL, *
 * INCIDENTAL, OR  INDIRECT DAMAGES OR FOR ANY ECONOMIC              *
 * CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS), EVEN   *
 * IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.  IBM  *
 * will not be liable for the loss of, or damage to, your records or *
 * data, or any damages claimed by you based on a third party claim. *
 *                                                                   *
 * IBM wishes to obtain your feedback to assist in improving the     *
 * Software.  You grant IBM a world-wide, royalty-free right to use, *
 * copy, distribute, sublicense and prepare derivative works based   *
 * upon any feedback, including materials, error corrections,        *
 * Software Derivatives, enhancements, suggestions and the like that *
 * you provide to IBM relating to the Software (this does not        *
 * include products for which you charge a royalty and distribute to *
 * IBM under other terms and conditions).                            *
 *                                                                   *
 * You agree to distribute the Software and any Software Derivatives *
 * under a license agreement that: 1) is sufficient to notify all    *
 * licensees of the Software and Software Derivatives that IBM       *
 * assumes no liability for any claim that may arise regarding the   *
 * Software or Software Derivatives, and 2) that disclaims all       *
 * warranties, both express and implied, from IBM regarding the      *
 * Software and Software Derivatives.  (If you include this          *
 * Agreement with any distribution of the Software or Software       *
 * Derivatives you will have met this requirement.)  You agree that  *
 * you will not delete any copyright notices in the Software.        *
 *                                                                   *
 * This Agreement is the exclusive statement of your rights in the   *
 * Software as provided by IBM.   Except for the rights granted to   *
 * you in the second paragraph above, You are not granted any other  *
 * patent rights, including but not limited to the right to make     *
 * combinations of the Software with products that infringe IBM      *
 * patents. You agree to comply with all applicable laws and         *
 * regulations, including all export and import laws and regulation. *
 * This Agreement is governed by the laws of the State of New York.  *
 * This Agreement supersedes all other communications,               *
 * understandings or agreements we may have had prior to this        *
 * Agreement.                                                        *
 * ***************************************************************** */

#ifndef JNH_TCP_H_
#define JNH_TCP_H_

// This file defines the low-level TCP service

#include <platform.h>

#ifdef _WIN32
#include <winsock2.h>
#endif


#include <commsg.h>

// An IPAddressV4 object encapsulates IPV4 addresses and the DNS.
class EXPORTCLASS ipAddressV4 {
protected:
  unsigned long addr;
public:
  bool ok;
  ipAddressV4(const char * name);
  ipAddressV4(unsigned char i1, unsigned char i2, unsigned char i3, unsigned char i4);
  ipAddressV4(unsigned long ip);
  ipAddressV4(void);
  ~ipAddressV4();
  uint32 setAddress(const char * name);
  uint32 setAddress(unsigned char i1, unsigned char i2, unsigned char i3, unsigned char i4);
  uint32 setAddress(unsigned long ip);
  uint32 getAddress(unsigned long & ip);
  uint32 getAddress(unsigned char & i1, unsigned char & i2, unsigned char & i3, unsigned char & i4);
};

// A TCP object represents a TCP connection.
class EXPORTCLASS TCP {
private:
  static bool wsi;
protected:
  int myPort;
#ifdef _WIN32
  SOCKET s;
  TCP(SOCKET s);
#endif
  bool listening;
public:
  TCP(void);
  ~TCP();

// In the event of an error, <error> contains the underlying socket error.
  int error;

// Binds the socket to a local port.
  uint32 setLocalPort(unsigned port);
// getLocalPort() returns the local port this socket is bound to, 
// causing one to be allocated if required.
  unsigned getLocalPort(void);  

// Active open.  Connects the socket to the specified host/port.
// If a local port hasn't been specified, one will be allocated
// here.
  uint32 open(ipAddressV4 host, unsigned port);

// Close the connection.
  uint32 close(void);

// Send and Rcv, with and without timeouts.  The timeout on send() applies to
// each network tranmission, rather than the send as a whole, so if the remote 
// end is pulling bytes slowly off the wire, send may take significantly longer
// than the timeout value.  Timeouts are specified in milliseconds.
  uint32 send(const unsigned char * data, size_t data_len);
  uint32 send(const unsigned char * data, size_t data_len, 
              unsigned timeout);
  uint32 rcv(unsigned char * buffer, size_t buf_len, size_t & data_len);
  uint32 rcv(unsigned char * buffer, size_t buf_len, size_t & data_len, 
             unsigned timeout);

// Listen() sets the socket to listen mode. You may specify a local
// port on the listen call only if you haven't already set one up
// with setLocalPort()
  uint32 listen(void);
  uint32 listen(unsigned localPort);
// Accept causes a listening socket to wait for connect attempts.  When an 
// incoming connection is received, a new TCP object will be created 
// to represent the accepted connection.  
  TCP * accept(void);

};

#endif